/*

HXED, Version 2.00, a Hexadecimal Editor for Binary Files
Copyright 1991 Anthony F. Stuart, All rights reserved.

Date    Name	Change
910209	AFS	Initial release

*/


#include <dos.h>
#include "sd.h"
#include "def.h"
#include "ext.h"

/* PC ROM BIOS Functions */

#define VIDEO_INT	16
#define SET_MODE	0
#define POSITION_CURSOR	2
#define SCROLL_UP	6
#define	SCROLL_DOWN	7
#define WRITE_CHAR	9
#define GET_MODE	15
#define MONO_TEXT	7

#define KEYBOARD_INT	22
#define GET_KEY		0

static char far		*video_base;
static int		 video_mode;
static int		 video_page = 0;
static int		 video_bytes_per_row = 160;
static int		 use_direct_video = TRUE;

/* Macro to calculate direct video position */

#define get_video_addr(r,c) (video_base + (r * video_bytes_per_row) + (c << 1))

/* Invoke the ROM BIOS scroll down function to */
/* scroll the screen down one line. */

scroll_screen_down ()
{
	union REGS	 inregs;
	union REGS	 outregs;

	inregs.h.ah = SCROLL_DOWN;
	inregs.h.al = 1;
	inregs.h.ch = text_row;
	inregs.h.cl = text_col;
	inregs.h.dh = text_maxrow;
	inregs.h.dl = text_maxcol;
	inregs.h.bh = text_attr;
	int86 (VIDEO_INT, &inregs, &outregs);
}

/* Invoke the ROM BIOS scroll up function to */
/* scroll the screen up one line. */

scroll_screen_up ()
{
	union REGS	 inregs;
	union REGS	 outregs;

	inregs.h.ah = SCROLL_UP;
	inregs.h.al = 1;
	inregs.h.ch = text_row;
	inregs.h.cl = text_col;
	inregs.h.dh = text_maxrow;
	inregs.h.dl = text_maxcol;
	inregs.h.bh = text_attr;
	int86 (VIDEO_INT, &inregs, &outregs);
}

/* Invoke the ROM BIOS scroll up function to */
/* clear the screen by scrolling it up 25 lines. */

clear_screen (attr)
int	 attr;
{
	union REGS	 inregs;
	union REGS	 outregs;

	inregs.h.ah = SCROLL_UP;
	inregs.h.al = 25;
	inregs.h.ch = minrow;
	inregs.h.cl = mincol;
	inregs.h.dh = maxrow;
	inregs.h.dl = maxcol;
	inregs.h.bh = attr;
	int86 (VIDEO_INT, &inregs, &outregs);
}

/* Invoke the ROM BIOS position and write functions to */
/* truncate the specified row in the specified attribute */

truncate_row (row, col, attr)
int	 row;
int	 col;
int	 attr;
{
	union REGS	 inregs;
	union REGS	 outregs;

	/* Position cursor to specified row and column */

	inregs.h.ah = POSITION_CURSOR;
	inregs.h.dh = row;
	inregs.h.dl = col;
	inregs.h.bh = video_page;
	int86 (VIDEO_INT, &inregs, &outregs);

	/* Overwrite remainder of row with blanks */

	inregs.h.ah = WRITE_CHAR;
	inregs.h.al = ' ';
	inregs.h.bl = attr;
	inregs.h.bh = video_page;
	inregs.x.cx = (maxcol-col)+1;
	int86 (VIDEO_INT, &inregs, &outregs);
}

/* Invoke the ROM BIOS cursor position function to */
/* position the cursor to the specified row and column */

position_cursor (row, col)
int	 row;
int	 col;
{
	union REGS	 inregs;
	union REGS	 outregs;

	inregs.h.ah = 2;
	inregs.h.dh = row;
	inregs.h.dl = col;
	inregs.h.bh = video_page;
	int86 (VIDEO_INT, &inregs, &outregs);
}

/* Invoke the display character function to */
/* display a string of characters. */

display_string (row, col, buf, attr)
int	 row;
int	 col;
char	*buf;
int	 attr;
{
	while (*buf) {
		display_char (row, col++, *buf++, attr);
	}
}

/* Use direct video memory access or invoke the ROM BIOS */
/* position cursor and write character functions to write */
/* a character to the display. */

/* The direct video approach is simple minded and does not */
/* take into account video modes other than 25x80 text. */

display_char (row, col, c, attr)
int	 row;
int	 col;
int	 c;
int	 attr;
{
	union REGS	 inregs;
	union REGS	 outregs;
	char far	*video_addr;

	if (use_direct_video) {
		video_addr = get_video_addr (row, col);
		*video_addr++ = c;
		*video_addr = attr;
		return;
	}

	inregs.h.ah = POSITION_CURSOR;
	inregs.h.dh = row;
	inregs.h.dl = col;
	inregs.h.bh = video_page;
	int86 (VIDEO_INT, &inregs, &outregs);

	inregs.h.ah = WRITE_CHAR;
	inregs.h.al = c;
	inregs.h.bl = attr;
	inregs.h.bh = video_page;
	inregs.x.cx = 1;
	int86 (VIDEO_INT, &inregs, &outregs);
}

/* Invoke the ROM BIOS read key function to wait until a key */
/* is entered and read it from the keyboard input buffer. */

get_keystroke ()
{
	int		 c;
	union REGS	 inregs;
	union REGS	 outregs;

	inregs.h.ah = GET_KEY;
	int86 (KEYBOARD_INT, &inregs, &outregs);
	return (outregs.x.ax);
}

#define SPEAKER_PORT	 97
#define SPEAKER_ON	 0x03

#define TIMER_DATA	 66
#define TIMER_CONTROL	 67
#define TIMER_ON	 182

#define CYCLE_TIME	 1193180
#define FREQUENCY	 360
#define DURATION	 80		/* milliseconds */

/* Configure the programmable timer and speaker to */
/* produce an audible alert. */

beep ()
{
	long	 clock;
	int	 count;
	int	 speaker_status;

	/* Return immediately if user has selected silent mode */

	if (silent_mode)
		return;

	/* Convert frequency to clock cycles */

	clock = CYCLE_TIME;
	count = clock / FREQUENCY;

	/* Program timer associated with speaker */

	outp (TIMER_CONTROL, TIMER_ON);
	outp (TIMER_DATA, count % 256);
	outp (TIMER_DATA, count / 256);

	/* Turn speaker on */

	speaker_status = inp (SPEAKER_PORT);
	outp (SPEAKER_PORT, speaker_status | SPEAKER_ON);

	/* Let tone sound for specified duration */

	wait (DURATION);

	/* Turn speaker off */

	speaker_status = inp (SPEAKER_PORT);
	outp (SPEAKER_PORT, speaker_status & ~SPEAKER_ON);
}

/* Return time in milliseconds since midnight */

long
get_time ()
{
	long		 ms;
	union REGS	 inregs;
	union REGS	 outregs;

	inregs.h.ah = 0x2c;				/* get time */
	int86 (0x21, &inregs, &outregs);		/* dos */
	ms  = ((long) outregs.h.ch * 1000 * 60 * 60);	/* hours */
	ms += ((long) outregs.h.cl * 1000 * 60);	/* minutes */
	ms += ((long) outregs.h.dh * 1000);		/* seconds */
	ms += ((long) outregs.h.dl * 10);		/* 100's of second */
	return (ms);					/* milliseconds */
}

/* Loop for specified number of milliseconds */

wait (ms_to_wait)
int	 ms_to_wait;
{
	long		 start_time;
	long		 ms_elapsed;

	start_time = get_time ();
	ms_elapsed = 0;

	while (ms_elapsed < ms_to_wait) {
		ms_elapsed = get_time () - start_time;
	}
	
}

/* Initialize hardware */

init_hardware ()
{
	union REGS	 inregs;
	union REGS	 outregs;

	/* Get current video mode */

	inregs.h.ah = GET_MODE;
	inregs.h.bh = (unsigned char) video_page;
	int86 (VIDEO_INT, &inregs, &outregs);
	
	/* Select video memory address based on mode */

	video_mode = outregs.h.al;
	if (video_mode == MONO_TEXT) {
		video_base = (char far *) 0xb0000000;
		header_attr = 0x70;		/* black on white */
		text_attr = 0x07;		/* white on black */
		cursor_attr = 0x70;		/* black on white */
		footer_attr = 0x70;		/* black on white */
		message_attr = 0x07;		/* white on black */
	}
	else {
		video_base = (char far *) 0xb8000000;
		header_attr = 0x4e;		/* yellow on red */
		text_attr = 0x0b;		/* cyan on black */
		cursor_attr = 0x70;		/* black on white */
		footer_attr = 0x4e;		/* yellow on red */
		message_attr = 0x07;		/* white on black */
	}

	/* Set default display character to centered dot */

	def_disp = 0xfa;
}

restore_hardware ()
{
}

set_direct_video (state)
int	 state;
{
	use_direct_video = state;
}

/* Convert a user supplied attribute string to internal */
/* foreground and background attribute codes */

get_attr (attr)
char	*attr;
{
	int	 foreground;
	int	 background;

	if (attr[0] == EOS || attr[1] == EOS)
		return (DEFAULT_ATTR);

	foreground = convert_attr (attr[0]);
	background = convert_attr (attr[1]);

	return ((background << 4) | foreground | HIGH_INTENSITY);
}

/* Convert an attribute character to its */
/* corresponding numeric value. */

convert_attr (c)
char	 c;
{
	switch (c) {
		case 'b':		/* blue */
			return (1);
		case 'g':		/* green */
			return (2);
		case 'c':		/* cyan */
			return (3);
		case 'r':		/* red */
			return (4);
		case 'm':		/* magenta */
			return (5);
		case 'y':		/* yellow */
			return (6);
		case 'w':		/* white */
			return (7);
		case 'k':		/* black */
		default:
			return (0);
	}
}

